@@ -1,14 +1,21 @@
+from __future__ import annotations
+
 # standard libraries
+import asyncio
 import functools
 import gettext
 import logging
+import logging.handlers
+import math
 import numpy
+import pkgutil
 import sys
 import time
+import typing
 
 # local libraries
+from nion.instrumentation import camera_base
 from nion.swift import DataItemThumbnailWidget
-from nion.swift import Decorators
 from nion.swift import DisplayPanel
 from nion.swift import Panel
 from nion.swift import Workspace
@@ -17,10 +24,16 @@ from nion.swift.model import HardwareSource
 from nion.swift.model import PlugInManager
 from nion.ui import CanvasItem
 from nion.ui import Declarative
+from nion.ui import Dialog
 from nion.ui import Widgets
+from nion.ui import Window
 from nion.utils import Geometry
 from nion.utils import Registry
 
+if typing.TYPE_CHECKING:
+    from nion.swift.model import DisplayItem
+
+
 _ = gettext.gettext
 
 
@@ -81,16 +94,14 @@ class CameraControlStateController:
         on_abort_button_state_changed(visible, enabled)
         on_monitor_button_state_changed(visible, enabled)
         on_capture_button_state_changed(visible, enabled)
-        on_display_data_item_changed(data_item)
         on_display_new_data_item(data_item)
-        on_processed_data_item_changed(data_item)
         on_log_messages(messages, data_elements)
         (thread) on_data_item_states_changed(data_item_states)
     """
 
     def __init__(self, camera_hardware_source, queue_task, document_model):
         self.__hardware_source = camera_hardware_source
-        self.__is_eels_camera = self.__hardware_source and self.__hardware_source.features.get("is_eels_camera", False)
+        self.__has_processed_channel = self.__hardware_source and self.__hardware_source.features.get("has_processed_channel", False)
         self.use_processed_data = False
         self.queue_task = queue_task
         self.__document_model = document_model
@@ -109,9 +120,7 @@ class CameraControlStateController:
         self.on_monitor_button_state_changed = None
         self.on_data_item_states_changed = None
         self.on_capture_button_state_changed = None
-        self.on_display_data_item_changed = None
         self.on_display_new_data_item = None
-        self.on_processed_data_item_changed = None
         self.on_camera_current_changed = None
         self.on_log_messages = None
 
@@ -121,27 +130,16 @@ class CameraControlStateController:
         self.__last_camera_current_time = 0
         self.__xdatas_available_event = self.__hardware_source.xdatas_available_event.listen(self.__receive_new_xdatas)
 
-        # this function is threadsafe
-        # it queues the threadsafe call to the UI thread, checking to make sure the
-        # hardware source wasn't closed before being called (mainly to make tests run).
-        def handle_data_item_changed():
-            def update_display_data_item():
-                if self.__hardware_source:
-                    self.__update_display_data_item()
-            self.queue_task(update_display_data_item)
-
-        self.__data_item = None
-        data_item_reference = document_model.get_data_item_reference(self.__hardware_source.hardware_source_id)
-        self.__data_item_changed_event_listener = data_item_reference.data_item_changed_event.listen(handle_data_item_changed)
-
-        self.__eels_data_item = None
-        eels_data_item_reference = document_model.get_data_item_reference(document_model.make_data_item_reference_key(self.__hardware_source.hardware_source_id, "summed"))
-        self.__eels_data_item_changed_event_listener = eels_data_item_reference.data_item_changed_event.listen(handle_data_item_changed)
+        self.data_item_reference = document_model.get_data_item_reference(self.__hardware_source.hardware_source_id)
+        self.processed_data_item_reference = document_model.get_data_item_reference(document_model.make_data_item_reference_key(self.__hardware_source.hardware_source_id, "summed"))
 
     def close(self):
         if self.__captured_xdatas_available_event:
             self.__captured_xdatas_available_event.close()
             self.__captured_xdatas_available_event = None
+        if self.__xdatas_available_event:
+            self.__xdatas_available_event.close()
+            self.__xdatas_available_event = None
         if self.__profile_changed_event_listener:
             self.__profile_changed_event_listener.close()
             self.__profile_changed_event_listener = None
@@ -157,10 +155,6 @@ class CameraControlStateController:
         if self.__log_messages_event_listener:
             self.__log_messages_event_listener.close()
             self.__log_messages_event_listener = None
-        self.__data_item_changed_event_listener.close()
-        self.__data_item_changed_event_listener = None
-        self.__eels_data_item_changed_event_listener.close()
-        self.__eels_data_item_changed_event_listener = None
         self.on_display_name_changed = None
         self.on_binning_values_changed = None
         self.on_profiles_changed = None
@@ -171,12 +165,13 @@ class CameraControlStateController:
         self.on_monitor_button_state_changed = None
         self.on_data_item_states_changed = None
         self.on_capture_button_state_changed = None
-        self.on_display_data_item_changed = None
         self.on_display_new_data_item = None
-        self.on_processed_data_item_changed = None
         self.on_log_messages = None
         self.__hardware_source = None
 
+    def _reset_camera_current(self):
+        self.__last_camera_current_time = 0
+
     def __update_play_button_state(self):
         enabled = self.__hardware_source is not None
         if self.on_play_button_state_changed:
@@ -215,21 +210,8 @@ class CameraControlStateController:
                 self.queue_task(functools.partial(update_profile_state_and_frame_parameters, name))
 
     @property
-    def has_processed_data(self):
-        return self.__is_eels_camera
-
-    # not thread safe
-    def __update_display_data_item(self):
-        data_item_reference = self.__document_model.get_data_item_reference(self.__hardware_source.hardware_source_id)
-        with data_item_reference.mutex:
-            self.__data_item = data_item_reference.data_item
-            if self.on_display_data_item_changed:
-                self.on_display_data_item_changed(self.__data_item)
-        eels_data_item_reference = self.__document_model.get_data_item_reference(self.__document_model.make_data_item_reference_key(self.__hardware_source.hardware_source_id, "summed"))
-        with eels_data_item_reference.mutex:
-            self.__eels_data_item = eels_data_item_reference.data_item
-            if self.on_processed_data_item_changed:
-                self.on_processed_data_item_changed(self.__eels_data_item)
+    def has_processed_data(self) -> bool:
+        return self.__has_processed_channel
 
     def __receive_new_xdatas(self, xdatas):
         current_time = time.time()
@@ -239,7 +221,7 @@ class CameraControlStateController:
             exposure = xdata.metadata.get("hardware_source", dict()).get("exposure")
             if xdata.intensity_calibration.units == "counts" and counts_per_electron and exposure:
                 sum_counts = xdata.intensity_calibration.convert_to_calibrated_value(numpy.sum(xdatas[0].data))
-                detector_current = sum_counts / exposure / counts_per_electron / 6.242e18
+                detector_current = sum_counts / exposure / counts_per_electron / 6.242e18 if exposure > 0 and counts_per_electron > 0 else 0.0
                 if detector_current != self.__camera_current:
                     self.__camera_current = detector_current
                     def update_camera_current():
@@ -269,7 +251,6 @@ class CameraControlStateController:
             self.__update_profile_index(self.__hardware_source.selected_profile_index)
         if self.on_data_item_states_changed:
             self.on_data_item_states_changed(list())
-        self.__update_display_data_item()
 
     # must be called on ui thread
     def handle_change_profile(self, profile_label):
@@ -300,16 +281,16 @@ class CameraControlStateController:
             self.__hardware_source.open_monitor()
 
     # must be called on ui thread
-    def handle_shift_click(self, hardware_source_id, mouse_position, camera_shape):
+    def handle_shift_click(self, hardware_source_id, mouse_position, camera_shape, logger: logging.Logger) -> bool:
         if hardware_source_id == self.__hardware_source.hardware_source_id:
-            self.__hardware_source.shift_click(mouse_position, camera_shape)
+            self.__hardware_source.shift_click(mouse_position, camera_shape, logger)
             return True
         return False
 
     # must be called on ui thread
-    def handle_tilt_click(self, hardware_source_id, mouse_position, camera_shape):
+    def handle_tilt_click(self, hardware_source_id, mouse_position, camera_shape, logger: logging.Logger) -> bool:
         if hardware_source_id == self.__hardware_source.hardware_source_id:
-            self.__hardware_source.tilt_click(mouse_position, camera_shape)
+            self.__hardware_source.tilt_click(mouse_position, camera_shape, logger)
             return True
         return False
 
@@ -568,6 +549,43 @@ class CharButtonCanvasItem(CanvasItem.TextButtonCanvasItem):
             drawing_context.fill_text(self.__char, center_x, center_y + text_base)
 
 
+class CameraPanelDelegate:
+
+    def has_feature(self, feature_flag: str) -> bool:
+        return False
+
+    def open_help(self, *, api_broker: PlugInManager.APIBroker = None) -> bool:
+        return False
+
+    def get_configuration_ui_handler(self, *, api_broker: PlugInManager.APIBroker = None,
+                                     event_loop: asyncio.AbstractEventLoop = None,
+                                     hardware_source_id: str = None,
+                                     camera_device: camera_base.CameraDevice = None,
+                                     camera_settings: camera_base.CameraSettings = None,
+                                     **kwargs):
+        return None
+
+    def open_configuration(self, *, api_broker: PlugInManager.APIBroker,
+                           hardware_source_id: str = None,
+                           camera_device: camera_base.CameraDevice = None,
+                           camera_settings: camera_base.CameraSettings = None) -> bool:
+        return False
+
+    def get_monitor_ui_handler(self, *, api_broker: PlugInManager.APIBroker = None,
+                               event_loop: asyncio.AbstractEventLoop = None,
+                               hardware_source_id: str = None,
+                               camera_device: camera_base.CameraDevice = None,
+                               camera_settings: camera_base.CameraSettings = None,
+                               **kwargs):
+        return None
+
+    def open_monitor(self, *, api_broker: PlugInManager.APIBroker,
+                     hardware_source_id: str = None,
+                     camera_device: camera_base.CameraDevice = None,
+                     camera_settings: camera_base.CameraSettings = None) -> bool:
+        return False
+
+
 class CameraControlWidget(Widgets.CompositeWidgetBase):
 
     def __init__(self, document_controller, camera_controller):
@@ -577,6 +595,13 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
 
         self.__state_controller = CameraControlStateController(camera_controller, document_controller.queue_task, document_controller.document_model)
 
+        self.__delegate : typing.Optional[CameraPanelDelegate] = None
+
+        camera_panel_delegate_type = camera_controller.features.get("camera_panel_delegate_type")
+        for component in Registry.get_components_by_type("camera_panel_delegate"):
+            if component.camera_panel_delegate_type == camera_panel_delegate_type:
+                self.__delegate = component
+
         self.__shift_click_state = None
 
         self.__changes_blocked = False
@@ -589,10 +614,89 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         self.__image_display_mouse_released_event_listener = DisplayPanel.DisplayPanelManager().image_display_mouse_released_event.listen(self.image_panel_mouse_released)
         self.__mouse_pressed = False
 
-        open_controls_button = CanvasItem.BitmapButtonCanvasItem(document_controller.ui.load_rgba_data_from_file(Decorators.relative_file(__file__, "resources/sliders_icon_24.png")))
-        open_controls_widget = ui.create_canvas_widget(properties={"height": 24, "width": 24})
-        open_controls_widget.canvas_item.add_canvas_item(open_controls_button)
-        monitor_button = ui.create_push_button_widget(_("Monitor View..."))
+        help_widget = None
+        if self.__delegate and self.__delegate.has_feature("help"):
+
+            def help_button_clicked() -> None:
+                api_broker = PlugInManager.APIBroker()
+                if self.__delegate:
+                    self.__delegate.open_help(api_broker=api_broker)
+
+            help_button = CanvasItem.BitmapButtonCanvasItem(CanvasItem.load_rgba_data_from_bytes(pkgutil.get_data(__name__, "resources/help_icon_24.png"), "png"))
+            help_button.on_button_clicked = help_button_clicked
+            help_widget = ui.create_canvas_widget(properties={"height": 24, "width": 24})
+            help_widget.canvas_item.add_canvas_item(help_button)
+
+        open_controls_widget = None
+        self.__configuration_dialog_close_listener = None
+        if not self.__delegate or self.__delegate.has_feature("configuration"):
+
+            def configuration_button_clicked() -> None:
+                api_broker = PlugInManager.APIBroker()
+                if self.__delegate:
+                    if self.__configuration_dialog_close_listener:
+                        return
+                    # if not already open, see if delegate wants to open it via a ui handler.
+                    ui_handler = self.__delegate.get_configuration_ui_handler(api_broker=api_broker,
+                                                                              event_loop=document_controller.event_loop,
+                                                                              hardware_source_id = camera_controller.hardware_source_id,
+                                                                              camera_device = camera_controller.camera,
+                                                                              camera_settings = camera_controller.camera_settings)
+                    if ui_handler:
+                        dialog = Dialog.ActionDialog(ui, camera_controller.display_name)
+                        dialog.content.add(Declarative.DeclarativeWidget(document_controller.ui, document_controller.event_loop, ui_handler))
+                        def wc(w): self.__configuration_dialog_close_listener = None
+                        self.__configuration_dialog_close_listener = dialog._window_close_event.listen(wc)
+                        dialog.show()
+                        return
+                    # fall through means there is no declarative configuration dialog
+                    if self.__delegate.open_configuration(api_broker=api_broker,
+                                                          hardware_source_id=camera_controller.hardware_source_id,
+                                                          camera_device=camera_controller.camera,
+                                                          camera_settings=camera_controller.camera_settings):
+                        return
+                # fall through: no ui handler or direct handler
+                self.__state_controller.handle_settings_button_clicked(api_broker)
+
+            open_controls_button = CanvasItem.BitmapButtonCanvasItem(CanvasItem.load_rgba_data_from_bytes(pkgutil.get_data(__name__, "resources/sliders_icon_24.png"), "png"))
+            open_controls_button.on_button_clicked = configuration_button_clicked
+            open_controls_widget = ui.create_canvas_widget(properties={"height": 24, "width": 24})
+            open_controls_widget.canvas_item.add_canvas_item(open_controls_button)
+
+        monitor_button = None
+        self.__monitor_dialog_close_listener = None
+        if not self.__delegate or self.__delegate.has_feature("monitor"):
+
+            def monitor_button_clicked() -> None:
+                api_broker = PlugInManager.APIBroker()
+                if self.__delegate:
+                    if self.__monitor_dialog_close_listener:
+                        return
+                    # if not already open, see if delegate wants to open it via a ui handler.
+                    ui_handler = self.__delegate.get_monitor_ui_handler(api_broker=api_broker,
+                                                                        event_loop=document_controller.event_loop,
+                                                                        hardware_source_id=camera_controller.hardware_source_id,
+                                                                        camera_device=camera_controller.camera,
+                                                                        camera_settings=camera_controller.camera_settings)
+                    if ui_handler:
+                        dialog = Dialog.ActionDialog(ui, document_controller)
+                        dialog.content.add(Declarative.DeclarativeWidget(document_controller.ui, document_controller.event_loop, ui_handler))
+                        def wc(w): self.__monitor_dialog_close_listener = None
+                        self.__monitor_dialog_close_listener = dialog._window_close_event.listen(wc)
+                        dialog.show()
+                        return
+                    # fall through means there is no declarative monitor dialog
+                    if self.__delegate.open_monitor(api_broker=api_broker,
+                                                    hardware_source_id=camera_controller.hardware_source_id,
+                                                    camera_device=camera_controller.camera,
+                                                    camera_settings=camera_controller.camera_settings):
+                        return
+                # fall through: no ui handler or direct handler
+                self.__state_controller.handle_monitor_button_clicked()
+
+            monitor_button = ui.create_push_button_widget(_("Monitor View..."))
+            monitor_button.on_clicked = monitor_button_clicked
+
         camera_current_label = ui.create_label_widget()
         profile_label = ui.create_label_widget(_("Mode: "), properties={"margin":4})
         profile_combo = ui.create_combo_box_widget(properties={"min-width":72})
@@ -604,7 +708,7 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
 
         document_model = self.document_controller.document_model
         data_item_reference = document_model.get_data_item_reference(camera_controller.hardware_source_id)
-        data_item_thumbnail_source = DataItemThumbnailWidget.DataItemReferenceThumbnailSource(ui, data_item_reference)
+        data_item_thumbnail_source = DataItemThumbnailWidget.DataItemReferenceThumbnailSource(ui, document_model, data_item_reference)
         thumbnail_widget = DataItemThumbnailWidget.DataItemThumbnailWidget(ui, data_item_thumbnail_source, Geometry.IntSize(width=48, height=48))
 
         def thumbnail_widget_drag(mime_data, thumbnail, hot_spot_x, hot_spot_y):
@@ -616,17 +720,21 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         button_row1.add(profile_label)
         button_row1.add(profile_combo)
         button_row1.add_stretch()
-        button_row1.add(open_controls_widget)
+        if help_widget:
+            button_row1.add(help_widget)
+        if open_controls_widget:
+            button_row1.add(open_controls_widget)
 
         button_row1a = ui.create_row_widget(properties={"spacing": 2})
-        button_row1a.add(monitor_button)
+        if monitor_button:
+            button_row1a.add(monitor_button)
         button_row1a.add_stretch()
         button_row1a.add(camera_current_label)
 
         def monitor_button_state_changed(visible, enabled):
-            monitor_button.visible = visible
-            monitor_button.enabled = enabled
-            button_row1a.visible = visible
+            if monitor_button:
+                monitor_button.visible = visible
+                monitor_button.enabled = enabled
 
         def binning_combo_text_changed(text):
             if not self.__changes_blocked:
@@ -719,8 +827,6 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
                 self.__state_controller.handle_change_profile(text)
                 profile_combo.request_refocus()
 
-        open_controls_button.on_button_clicked = functools.partial(self.__state_controller.handle_settings_button_clicked, PlugInManager.APIBroker())
-        monitor_button.on_clicked = self.__state_controller.handle_monitor_button_clicked
         profile_combo.on_current_text_changed = profile_combo_text_changed
 
         def binning_values_changed(binning_values):
@@ -747,7 +853,7 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
             blocked = self.__changes_blocked
             self.__changes_blocked = True
             try:
-                exposure_field.text = str("{0:.1f}".format(float(frame_parameters.exposure_ms)))
+                exposure_field.text = str("{0:.2f}".format(float(frame_parameters.exposure_ms)))
                 if exposure_field.focused:
                     exposure_field.request_refocus()
                 binning_combo.current_text = str(frame_parameters.binning)
@@ -775,7 +881,8 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
 
         def camera_current_changed(camera_current):
             if camera_current:
-                camera_current_label.text = str(int(camera_current * 1e12)) + _("pA")
+                camera_current_int = int(camera_current * 1e12) if math.isfinite(camera_current) else 0
+                camera_current_label.text = str(camera_current_int) + _("pA")
                 camera_current_label.text_color = "black"
             else:
                 camera_current_label.text_color = "gray"
@@ -808,6 +915,8 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         super().periodic()
 
     def close(self):
+        self.__configuration_dialog_close_listener = None
+        self.__monitor_dialog_close_listener = None
         self.__key_pressed_event_listener.close()
         self.__key_pressed_event_listener = None
         self.__key_released_event_listener.close()
@@ -822,22 +931,34 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
 
     # this gets called from the DisplayPanelManager. pass on the message to the state controller.
     # must be called on ui thread
-    def image_panel_mouse_pressed(self, display_panel, display_specifier, image_position, modifiers):
-        data_item = display_specifier.data_item if display_specifier else None
+    def image_panel_mouse_pressed(self, display_panel: DisplayPanel.DisplayPanel, display_item: DisplayItem.DisplayItem, image_position: Geometry.FloatPoint, modifiers: CanvasItem.KeyboardModifiers) -> bool:
+        data_item = display_panel.data_item if display_panel else None
         hardware_source_id = data_item and data_item.metadata.get("hardware_source", dict()).get("hardware_source_id")
+        logger = logging.getLogger("camera_control_ui")
+        logger.propagate = False  # do not send messages to root logger
+        if not logger.handlers:
+            logger.addHandler(logging.handlers.BufferingHandler(4))
         if self.__shift_click_state == "shift":
             mouse_position = image_position
             camera_shape = data_item.dimensional_shape
-            self.__mouse_pressed = self.__state_controller.handle_shift_click(hardware_source_id, mouse_position, camera_shape)
+            self.__mouse_pressed = self.__state_controller.handle_shift_click(hardware_source_id, mouse_position, camera_shape, logger)
+            logger_buffer = typing.cast(logging.handlers.BufferingHandler, logger.handlers[0])
+            for record in logger_buffer.buffer:
+                display_panel.document_controller.display_log_record(record)
+            logger_buffer.flush()
             return self.__mouse_pressed
         if self.__shift_click_state == "tilt":
             mouse_position = image_position
             camera_shape = data_item.dimensional_shape
-            self.__mouse_pressed = self.__state_controller.handle_tilt_click(hardware_source_id, mouse_position, camera_shape)
+            self.__mouse_pressed = self.__state_controller.handle_tilt_click(hardware_source_id, mouse_position, camera_shape, logger)
+            logger_buffer = typing.cast(logging.handlers.BufferingHandler, logger.handlers[0])
+            for record in logger_buffer.buffer:
+                display_panel.document_controller.display_log_record(record)
+            logger_buffer.flush()
             return self.__mouse_pressed
         return False
 
-    def image_panel_mouse_released(self, display_panel, display_specifier, image_position, modifiers):
+    def image_panel_mouse_released(self, display_panel: DisplayPanel.DisplayPanel, display_item: DisplayItem.DisplayItem, image_position: Geometry.FloatPoint, modifiers: CanvasItem.KeyboardModifiers) -> bool:
         mouse_pressed = self.__mouse_pressed
         self.__mouse_pressed = False
         return mouse_pressed
@@ -855,6 +976,10 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         self.__shift_click_state = None
         return False
 
+    @property
+    def state_controller(self):
+        return self.__state_controller
+
 
 class CameraControlPanel(Panel.Panel):
 
@@ -938,8 +1063,6 @@ class CameraDisplayPanelController:
         capture_button = CanvasItem.TextButtonCanvasItem(_("Capture"))
         capture_button.border_enabled = False
         playback_controls_row.add_canvas_item(capture_button)
-        self.__last_data_item = None
-        self.__last_processed_data_item = None
         self.__show_processed_checkbox = None
         if self.__state_controller.has_processed_data:
             self.__show_processed_checkbox = CanvasItem.CheckBoxCanvasItem()
@@ -1015,23 +1138,13 @@ class CameraDisplayPanelController:
                 capture_button.text = str()
                 capture_button.size_to_content(display_panel.image_panel_get_font_metrics)
 
-        def display_data_item_changed(data_item):
-            if not self.__show_processed_checkbox or not self.__show_processed_checkbox.check_state == "checked":
-                self.__state_controller.use_processed_data = False  # for capture
-                display_panel.set_displayed_data_item(data_item)
-            self.__last_data_item = data_item
-
-        def processed_data_item_changed(data_item):
-            if self.__show_processed_checkbox and self.__show_processed_checkbox.check_state == "checked":
-                self.__state_controller.use_processed_data = True  # for capture
-                display_panel.set_displayed_data_item(data_item)
-            self.__last_processed_data_item = data_item
-
         def show_processed_checkbox_changed(check_state):
             if check_state == "checked":
-                processed_data_item_changed(self.__last_processed_data_item)
+                display_panel.set_data_item_reference(self.__state_controller.processed_data_item_reference)
+                self.__state_controller.use_processed_data = True  # for capture
             else:
-                display_data_item_changed(self.__last_data_item)
+                display_panel.set_data_item_reference(self.__state_controller.data_item_reference)
+                self.__state_controller.use_processed_data = False  # for capture
 
         def display_new_data_item(data_item):
             result_display_panel = display_panel.document_controller.next_result_display_panel()
@@ -1048,9 +1161,7 @@ class CameraDisplayPanelController:
         self.__state_controller.on_abort_button_state_changed = abort_button_state_changed
         self.__state_controller.on_data_item_states_changed = data_item_states_changed
         self.__state_controller.on_capture_button_state_changed = update_capture_button
-        self.__state_controller.on_display_data_item_changed = display_data_item_changed
         self.__state_controller.on_display_new_data_item = display_new_data_item
-        self.__state_controller.on_processed_data_item_changed = processed_data_item_changed
 
         play_button_canvas_item.on_button_clicked = self.__state_controller.handle_play_pause_clicked
         abort_button_canvas_item.on_button_clicked = self.__state_controller.handle_abort_clicked
@@ -1058,6 +1169,11 @@ class CameraDisplayPanelController:
 
         self.__state_controller.initialize_state()
 
+        checkstate = self.__show_processed_checkbox.check_state if self.__show_processed_checkbox else "unchecked"
+
+        show_processed_checkbox_changed(checkstate)
+
+
     def close(self):
         self.__display_panel.footer_canvas_item.remove_canvas_item(self.__playback_controls_composition)
         self.__display_panel = None
@@ -1081,6 +1197,10 @@ class CameraDisplayPanelController:
     def key_released(self, key):
         return False
 
+    @property
+    def hardware_source_id(self):
+        return self.__hardware_source_id
+
 
 hardware_source_added_event_listener = None
 hardware_source_removed_event_listener = None
@@ -1098,9 +1218,8 @@ def run():
         """Called when a hardware source is added to the hardware source manager."""
 
         # check to see if we handle this hardware source.
-        is_ronchigram_camera = hardware_source.features.get("is_ronchigram_camera", False)
-        is_eels_camera = hardware_source.features.get("is_eels_camera", False)
-        if is_ronchigram_camera or is_eels_camera:
+        is_camera = hardware_source.features.get("is_camera", False)
+        if is_camera:
 
             panel_id = "camera-control-panel-" + hardware_source.hardware_source_id
             name = hardware_source.display_name + " " + _("Camera Control")
@@ -1117,7 +1236,7 @@ def run():
                         selected_display_panel.change_display_panel_content(d)
 
                     action = display_type_menu.add_menu_item(hardware_source.display_name, functools.partial(switch_to_live_controller, hardware_source))
-                    action.checked = False
+                    action.checked = isinstance(selected_display_panel.display_panel_controller, CameraDisplayPanelController) and selected_display_panel.display_panel_controller.hardware_source_id == hardware_source.hardware_source_id
                     return [action]
 
                 def make_new(self, controller_type, display_panel, d):
@@ -1130,7 +1249,7 @@ def run():
                         return CameraDisplayPanelController(display_panel, hardware_source_id, show_processed_data)
                     return None
 
-                def match(self, document_model, data_item: DataItem.DataItem) -> dict:
+                def match(self, document_model, data_item: DataItem.DataItem) -> typing.Optional[typing.Dict]:
                     if HardwareSource.matches_hardware_source(hardware_source.hardware_source_id, None, document_model, data_item):
                         return {"controller_type": CameraDisplayPanelController.type, "hardware_source_id": hardware_source.hardware_source_id}
                     return None
@@ -1140,7 +1259,7 @@ def run():
             panel_properties = {"hardware_source_id": hardware_source.hardware_source_id}
 
             camera_panel_type = hardware_source.features.get("camera_panel_type")
-            if not camera_panel_type or camera_panel_type in ("ronchigram", "eels"):
+            if not camera_panel_type:
                 Workspace.WorkspaceManager().register_panel(CameraControlPanel, panel_id, name, ["left", "right"], "left", panel_properties)
             else:
                 panel_properties["camera_panel_type"] = camera_panel_type
@@ -1148,9 +1267,8 @@ def run():
 
     def unregister_camera_panel(hardware_source):
         """Called when a hardware source is removed from the hardware source manager."""
-        is_ronchigram_camera = hardware_source.features.get("is_ronchigram_camera", False)
-        is_eels_camera = hardware_source.features.get("is_eels_camera", False)
-        if is_ronchigram_camera or is_eels_camera:
+        is_camera = hardware_source.features.get("is_camera", False)
+        if is_camera:
             DisplayPanel.DisplayPanelManager().unregister_display_panel_controller_factory("camera-live-" + hardware_source.hardware_source_id)
             panel_id = camera_control_panels.get(hardware_source.hardware_source_id)
             if panel_id:
